@@ -21,7 +21,14 @@ select_on_class <- function(.data, clases = "numeric") {
 
 #' contr.dummy
 #'
-#' @keywords internal
+#' @description Returns a matrix of contrasts for the \code{\link[kknn]{train.kknn}}.
+#'
+#' @param n A vector containing levels of a factor, or the number of levels.
+#' @param contrasts A logical value indicating whether contrasts should be computed.
+#'
+#' @return A matrix with n rows and n-1 columns for contr.ordinal, a matrix with n rows and n columns for contr.dummy and a vector of length n for contr.metric.
+#'
+#' @export contr.dummy
 #'
 contr.dummy <- function (n, contrasts = TRUE) {
   if (length(n) <= 1) {
@@ -38,7 +45,14 @@ contr.dummy <- function (n, contrasts = TRUE) {
 
 #' contr.ordinal
 #'
-#' @keywords internal
+#' @description Returns a matrix of contrasts for the \code{\link[kknn]{train.kknn}}.
+#'
+#' @param n A vector containing levels of a factor, or the number of levels.
+#' @param contrasts A logical value indicating whether contrasts should be computed.
+#'
+#' @return A matrix with n rows and n-1 columns for contr.ordinal, a matrix with n rows and n columns for contr.dummy and a vector of length n for contr.metric.
+#'
+#' @export contr.ordinal
 #'
 contr.ordinal <- function (n, contrasts = TRUE) {
   if (length(n) <= 1) {
@@ -54,6 +68,30 @@ contr.ordinal <- function (n, contrasts = TRUE) {
   cont
 }
 
+#' contr.metric
+#'
+#' @description Returns a matrix of contrasts for the \code{\link[kknn]{train.kknn}}.
+#'
+#' @param n A vector containing levels of a factor, or the number of levels.
+#' @param contrasts A logical value indicating whether contrasts should be computed.
+#'
+#' @return A matrix with n rows and n-1 columns for contr.ordinal, a matrix with n rows and n columns for contr.dummy and a vector of length n for contr.metric.
+#'
+#' @export contr.metric
+#'
+contr.metric <- function (n, contrasts = TRUE) {
+  if (length(n) <= 1) {
+    if (is.numeric(n) && length(n) == 1 && n > 1)
+      levels <- 1:n
+    else stop("contrasts are not defined for 0 degrees of freedom")
+  }
+  else levels <- n
+  lenglev <- length(levels)
+  cont <- array((1:lenglev) - (1 + lenglev)/2, c(lenglev, 1),
+                list(levels, NULL))
+  cont
+}
+
 #' max_col
 #'
 #' @keywords internal
@@ -158,14 +196,18 @@ print.prediction.prmdt <- function(x, ...){
 #' @export print.indexes.prmdt
 #' @export
 #'
-print.indexes.prmdt <- function(x, ...){
-  out <- c("\nConfusion Matrix:",capture.output(x$confusion.matrix))
-  out <- paste(out, collapse = "\n")
-  out <- paste0(out, sprintf("\n\nOverall Accuracy: %3.4f\nOverall Error: %9.4f\n", x$overall.accuracy, x$overall.error))
-  out.aux <- do.call(sprintf, c(paste0(rep("%13s", length(x$category.accuracy)), collapse = ""), as.list(names(x$category.accuracy))))
-  out.aux <- paste0("\nCategory Accuracy:\n\n",out.aux, "\n",
-                    do.call(sprintf, c(paste0(rep("%13f", length(x$category.accuracy)), collapse = ""), as.list(as.numeric(x$category.accuracy)))))
-  out <- paste0(out, out.aux)
+print.indexes.prmdt <- function(x, ...) {
+  if(is.null(x$confusion.matrix)) {
+    out <- paste(capture.output(unlist(x)), collapse = "\n")
+  } else {
+    out <- c("\nConfusion Matrix:",capture.output(x$confusion.matrix))
+    out <- paste(out, collapse = "\n")
+    out <- paste0(out, sprintf("\n\nOverall Accuracy: %3.4f\nOverall Error: %9.4f\n", x$overall.accuracy, x$overall.error))
+    out.aux <- do.call(sprintf, c(paste0(rep("%13s", length(x$category.accuracy)), collapse = ""), as.list(names(x$category.accuracy))))
+    out.aux <- paste0("\nCategory Accuracy:\n\n",out.aux, "\n",
+                      do.call(sprintf, c(paste0(rep("%13f", length(x$category.accuracy)), collapse = ""), as.list(as.numeric(x$category.accuracy)))))
+    out <- paste0(out, out.aux)
+  }
   cat(out)
 }
 
@@ -198,6 +240,49 @@ varplot <- function(x, ...){
   ada::varplot(x, ...)
 }
 
+#' dummy
+#'
+#' function from dummies package
+#'
+#' @keywords internal
+#'
+dummy <- function (x, data = NULL, sep = "", drop = TRUE, fun = as.integer,
+                   verbose = FALSE)
+{
+  if (is.null(data)) {
+    name <- as.character(sys.call(1))[2]
+    name <- sub("^(.*\\$)", "", name)
+    name <- sub("\\[.*\\]$", "", name)
+  }
+  else {
+    if (length(x) > 1)
+      stop("More than one variable provided to produce dummy variable.")
+    name <- x
+    x <- data[, name]
+  }
+  if (drop == FALSE && is.factor(x)) {
+    x <- factor(x, levels = levels(x), exclude = NULL)
+  } else {
+    x <- factor(x, exclude = NULL)
+  }
+  if (length(levels(x)) < 2) {
+    if (verbose)
+      warning(name, " has only 1 level. Producing dummy variable anyway.")
+    return(matrix(rep(1, length(x)), ncol = 1, dimnames = list(rownames(x),
+                                                               c(paste(name, sep, x[[1]], sep = "")))))
+  }
+  mm <- model.matrix(~x - 1, model.frame(~x - 1), contrasts = FALSE)
+  colnames.mm <- colnames(mm)
+  if (verbose)
+    cat(" ", name, ":", ncol(mm), "dummy varibles created\n")
+  mm <- matrix(fun(mm), nrow = nrow(mm), ncol = ncol(mm), dimnames = list(NULL,
+                                                                          colnames.mm))
+  colnames(mm) <- sub("^x", paste(name, sep, sep = ""), colnames(mm))
+  if (!is.null(row.names(data)))
+    rownames(mm) <- rownames(data)
+  return(mm)
+}
+
 #' dummy.data.frame
 #'
 #' function from dummies package
